Package: ape
Version: 5.4-1
Date: 2020-08-12
Title: Analyses of Phylogenetics and Evolution
Authors@R: c(person("Emmanuel", "Paradis", role = c("aut", "cre", "cph"), email = "Emmanuel.Paradis@ird.fr", comment = c(ORCID = "0000-0003-3092-2199")),
  person("Simon", "Blomberg", role = c("aut", "cph"), comment = c(ORCID = "0000-0003-1062-0839")),
  person("Ben", "Bolker", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-2127-0443")),
  person("Joseph", "Brown", role = c("aut", "cph")),
  person("Santiago", "Claramunt", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-8926-5974")),
  person("Julien", "Claude", role = c("aut", "cph"), , comment = c(ORCID = "0000-0002-9267-1228")),
  person("Hoa Sien", "Cuong", role = c("aut", "cph")),
  person("Richard", "Desper", role = c("aut", "cph")),
  person("Gilles", "Didier", role = c("aut", "cph"), comment = c(ORCID = "0000-0003-0596-9112")),
  person("Benoit", "Durand", role = c("aut", "cph")),
  person("Julien", "Dutheil", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-7753-4121")),
  person("RJ", "Ewing", role = c("aut", "cph")),
  person("Olivier", "Gascuel", role = c("aut", "cph")),
  person("Thomas", "Guillerme", role = c("aut", "cph"), comment = c(ORCID = "0000-0003-4325-1275")),
  person("Christoph", "Heibl", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-7655-3299")),
  person("Anthony", "Ives", role = c("aut", "cph")),
  person("Bradley", "Jones", role = c("aut", "cph"), comment = c(ORCID = "0000-0003-4498-1069")),
  person("Franz", "Krah", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-7866-7508")),
  person("Daniel", "Lawson", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-5311-6213")),
  person("Vincent", "Lefort", role = c("aut", "cph")),
  person("Pierre", "Legendre", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-3838-3305")),
  person("Jim", "Lemon", role = c("aut", "cph")),
  person("Guillaume", "Louvel", role = c("aut", "cph")),
  person("Eric", "Marcon", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-5249-321X")),
  person("Rosemary", "McCloskey", role = c("aut", "cph")),
  person("Johan", "Nylander", role = c("aut", "cph")),
  person("Rainer", "Opgen-Rhein", role = c("aut", "cph")),
  person("Andrei-Alin", "Popescu", role = c("aut", "cph")),
  person("Manuela", "Royer-Carenzi", role = c("aut", "cph")),
  person("Klaus", "Schliep", role = c("aut", "cph"), comment = c(ORCID = "0000-0003-2941-0161")),
  person("Korbinian", "Strimmer", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-7917-2056")),
  person("Damien", "de Vienne", role = c("aut", "cph"), comment = c(ORCID = "0000-0001-9532-5251")))
Depends: R (>= 3.2.0)
Suggests: gee, expm, igraph
Imports: nlme, lattice, graphics, methods, stats, tools, utils,
        parallel, Rcpp (>= 0.12.0)
LinkingTo: Rcpp
ZipData: no
Description: Functions for reading, writing, plotting, and manipulating phylogenetic trees, analyses of comparative data in a phylogenetic framework, ancestral character analyses, analyses of diversification and macroevolution, computing distances from DNA sequences, reading and writing nucleotide sequences as well as importing from BioConductor, and several tools such as Mantel's test, generalized skyline plots, graphical exploration of phylogenetic data (alex, trex, kronoviz), estimation of absolute evolutionary rates and clock-like trees using mean path lengths and penalized likelihood, dating trees with non-contemporaneous sequences, translating DNA into AA sequences, and assessing sequence alignments. Phylogeny estimation can be done with the NJ, BIONJ, ME, MVR, SDM, and triangle methods, and several methods handling incomplete distance matrices (NJ*, BIONJ*, MVR*, and the corresponding triangle method). Some functions call external applications (PhyML, Clustal, T-Coffee, Muscle) whose results are returned into R.
License: GPL-2 | GPL-3
URL: http://ape-package.ird.fr/
NeedsCompilation: yes
Packaged: 2020-08-12 09:57:49 UTC; paradis
Author: Emmanuel Paradis [aut, cre, cph]
    (<https://orcid.org/0000-0003-3092-2199>),
  Simon Blomberg [aut, cph] (<https://orcid.org/0000-0003-1062-0839>),
  Ben Bolker [aut, cph] (<https://orcid.org/0000-0002-2127-0443>),
  Joseph Brown [aut, cph],
  Santiago Claramunt [aut, cph] (<https://orcid.org/0000-0002-8926-5974>),
  Julien Claude [aut, cph] (<https://orcid.org/0000-0002-9267-1228>),
  Hoa Sien Cuong [aut, cph],
  Richard Desper [aut, cph],
  Gilles Didier [aut, cph] (<https://orcid.org/0000-0003-0596-9112>),
  Benoit Durand [aut, cph],
  Julien Dutheil [aut, cph] (<https://orcid.org/0000-0001-7753-4121>),
  RJ Ewing [aut, cph],
  Olivier Gascuel [aut, cph],
  Thomas Guillerme [aut, cph] (<https://orcid.org/0000-0003-4325-1275>),
  Christoph Heibl [aut, cph] (<https://orcid.org/0000-0002-7655-3299>),
  Anthony Ives [aut, cph],
  Bradley Jones [aut, cph] (<https://orcid.org/0000-0003-4498-1069>),
  Franz Krah [aut, cph] (<https://orcid.org/0000-0001-7866-7508>),
  Daniel Lawson [aut, cph] (<https://orcid.org/0000-0002-5311-6213>),
  Vincent Lefort [aut, cph],
  Pierre Legendre [aut, cph] (<https://orcid.org/0000-0002-3838-3305>),
  Jim Lemon [aut, cph],
  Guillaume Louvel [aut, cph],
  Eric Marcon [aut, cph] (<https://orcid.org/0000-0002-5249-321X>),
  Rosemary McCloskey [aut, cph],
  Johan Nylander [aut, cph],
  Rainer Opgen-Rhein [aut, cph],
  Andrei-Alin Popescu [aut, cph],
  Manuela Royer-Carenzi [aut, cph],
  Klaus Schliep [aut, cph] (<https://orcid.org/0000-0003-2941-0161>),
  Korbinian Strimmer [aut, cph] (<https://orcid.org/0000-0001-7917-2056>),
  Damien de Vienne [aut, cph] (<https://orcid.org/0000-0001-9532-5251>)
Maintainer: Emmanuel Paradis <Emmanuel.Paradis@ird.fr>
Repository: CRAN
Date/Publication: 2020-08-13 05:20:14 UTC
