# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bipartition2 <- function(orig, nTips) {
    .Call(`_ape_bipartition2`, orig, nTips)
}

prop_part2 <- function(trees, nTips) {
    .Call(`_ape_prop_part2`, trees, nTips)
}

reorderRcpp <- function(orig, nTips, root, order) {
    .Call(`_ape_reorderRcpp`, orig, nTips, root, order)
}

